@@ -12,8 +12,8 @@
  *                         All rights reserved.
  * Copyright (c) 2006-2007 Voltaire. All rights reserved.
  * Copyright (c) 2009-2010 Cisco Systems, Inc.  All rights reserved.
- * Copyright (c) 2010-2015 Los Alamos National Security, LLC.
- *                         All rights reserved.
+ * Copyright (c) 2010-2016 Los Alamos National Security, LLC. All rights
+ *                         reserved.
  * Copyright (c) 2011-2015 NVIDIA Corporation.  All rights reserved.
  * Copyright (c) 2014      Intel, Inc. All rights reserved.
  * $COPYRIGHT$
@@ -138,6 +138,13 @@ static int mca_btl_smcuda_component_verify(void) {
 static int smcuda_register(void)
 {
     /* register SM component parameters */
+    mca_btl_smcuda_component.mpool_min_size = 134217728;
+    (void) mca_base_component_var_register(&mca_btl_smcuda_component.super.btl_version, "min_size",
+                                           "Minimum size of the common/sm mpool shared memory file",
+                                           MCA_BASE_VAR_TYPE_UNSIGNED_LONG, NULL, 0, 0,
+                                           OPAL_INFO_LVL_9, MCA_BASE_VAR_SCOPE_READONLY,
+                                           &mca_btl_smcuda_component.mpool_min_size);
+
     mca_btl_smcuda_param_register_int("free_list_num", 8, OPAL_INFO_LVL_5, &mca_btl_smcuda_component.sm_free_list_num);
     mca_btl_smcuda_param_register_int("free_list_max", -1, OPAL_INFO_LVL_5, &mca_btl_smcuda_component.sm_free_list_max);
     mca_btl_smcuda_param_register_int("free_list_inc", 64, OPAL_INFO_LVL_5, &mca_btl_smcuda_component.sm_free_list_inc);
@@ -153,6 +160,12 @@ static int smcuda_register(void)
     /* default number of extra procs to allow for future growth */
     mca_btl_smcuda_param_register_int("sm_extra_procs", 0, OPAL_INFO_LVL_9, &mca_btl_smcuda_component.sm_extra_procs);
 
+    mca_btl_smcuda_component.allocator = "bucket";
+    (void) mca_base_component_var_register (&mca_btl_smcuda_component.super.btl_version, "allocator",
+                                            "Name of allocator component to use for btl/smcuda allocations",
+                                            MCA_BASE_VAR_TYPE_STRING, NULL, 0, 0, OPAL_INFO_LVL_9,
+                                            MCA_BASE_VAR_SCOPE_LOCAL, &mca_btl_smcuda_component.allocator);
+
 #if OPAL_CUDA_SUPPORT
     /* Lower priority when CUDA support is not requested */
     if (opal_cuda_support) {
@@ -353,41 +366,6 @@ create_and_attach(mca_btl_smcuda_component_t *comp_ptr,
     return OPAL_SUCCESS;
 }
 
-/*
- * SKG - I'm not happy with this, but I can't figure out a better way of
- * finding the sm mpool's minimum size 8-|. The way I see it. This BTL only
- * uses the sm mpool, so maybe this isn't so bad...
- *
- * The problem is the we need to size the mpool resources at sm BTL component
- * init. That means we need to know the mpool's minimum size at create.
- */
-static int
-get_min_mpool_size(mca_btl_smcuda_component_t *comp_ptr,
-                   size_t *out_size)
-{
-    const char *type_name = "mpool";
-    const char *param_name = "min_size";
-    const mca_base_var_storage_t *min_size;
-    int id = 0;
-
-    if (0 > (id = mca_base_var_find("ompi", type_name, comp_ptr->sm_mpool_name,
-                                      param_name))) {
-        opal_output(0, "mca_base_var_find: failure looking for %s_%s_%s\n",
-                    type_name, comp_ptr->sm_mpool_name, param_name);
-        return OPAL_ERR_NOT_FOUND;
-    }
-
-    if (OPAL_SUCCESS != mca_base_var_get_value(id, &min_size, NULL, NULL)) {
-        opal_output(0, "mca_base_var_get_value failure\n");
-        return OPAL_ERROR;
-    }
-
-    /* the min_size variable is an unsigned long long */
-    *out_size = (size_t) min_size->ullval;
-
-    return OPAL_SUCCESS;
-}
-
 static int
 get_mpool_res_size(int32_t max_procs,
                    size_t *out_res_size)
@@ -508,21 +486,18 @@ create_rndv_file(mca_btl_smcuda_component_t *comp_ptr,
     mca_common_sm_module_t *tmp_modp = NULL;
 
     if (MCA_BTL_SM_RNDV_MOD_MPOOL == type) {
-        size_t min_size = 0;
         /* get the segment size for the sm mpool. */
         if (OPAL_SUCCESS != (rc = get_mpool_res_size(comp_ptr->sm_max_procs,
                                                      &size))) {
             /* rc is already set */
             goto out;
         }
-        /* do we need to update the size based on the sm mpool's min size? */
-        if (OPAL_SUCCESS != (rc = get_min_mpool_size(comp_ptr, &min_size))) {
-            goto out;
-        }
+
         /* update size if less than required minimum */
-        if (size < min_size) {
-            size = min_size;
+        if (size < mca_btl_smcuda_component.mpool_min_size) {
+            size = mca_btl_smcuda_component.mpool_min_size;
         }
+
         /* we only need the shmem_ds info at this point. initilization will be
          * completed in the mpool module code. the idea is that we just need this
          * info so we can populate the rndv file (or modex when we have it). */
@@ -1148,8 +1123,8 @@ int mca_btl_smcuda_component_progress(void)
                 OPAL_SUCCESS);
 
         if(frag->registration != NULL) {
-            frag->endpoint->mpool->mpool_deregister(frag->endpoint->mpool,
-                                                    (mca_mpool_base_registration_t*)frag->registration);
+            frag->endpoint->rcache->rcache_deregister (frag->endpoint->rcache,
+                                                       (mca_rcache_base_registration_t*)frag->registration);
             frag->registration = NULL;
             MCA_BTL_SMCUDA_FRAG_RETURN(frag);
         }
