@@ -12,7 +12,7 @@
  *                         All rights reserved.
  * Copyright (c) 2006-2007 Voltaire. All rights reserved.
  * Copyright (c) 2009      Cisco Systems, Inc.  All rights reserved.
- * Copyright (c) 2010-2016 Los Alamos National Security, LLC. All rights
+ * Copyright (c) 2010-2015 Los Alamos National Security, LLC. All rights
  *                         reserved.
  * Copyright (c) 2014-2015 Intel, Inc. All rights reserved.
  * Copyright (c) 2014-2015 Research Organization for Information Science
@@ -145,6 +145,12 @@ static int vader_btl_first_time_init(mca_btl_vader_t *vader_btl, int n)
     /* set flag indicating btl has been inited */
     vader_btl->btl_inited = true;
 
+#if OPAL_BTL_VADER_HAVE_XPMEM
+    if (MCA_BTL_VADER_XPMEM == mca_btl_vader_component.single_copy_mechanism) {
+        mca_btl_vader_component.vma_module = mca_rcache_base_vma_module_alloc ();
+    }
+#endif
+
     return OPAL_SUCCESS;
 }
 
@@ -171,9 +177,8 @@ static int init_vader_endpoint (struct mca_btl_base_endpoint_t *ep, struct opal_
         if (MCA_BTL_VADER_XPMEM == mca_btl_vader_component.single_copy_mechanism) {
             /* always use xpmem if it is available */
             ep->segment_data.xpmem.apid = xpmem_get (modex->xpmem.seg_id, XPMEM_RDWR, XPMEM_PERMIT_MODE, (void *) 0666);
-            ep->segment_data.xpmem.rcache = mca_rcache_base_module_create("vma");
             (void) vader_get_registation (ep, modex->xpmem.segment_base, mca_btl_vader_component.segment_size,
-                                          MCA_MPOOL_FLAGS_PERSIST, (void **) &ep->segment_base);
+                                          MCA_RCACHE_FLAGS_PERSIST, (void **) &ep->segment_base);
         } else {
 #endif
             /* store a copy of the segment information for detach */
@@ -354,6 +359,12 @@ static int vader_finalize(struct mca_btl_base_module_t *btl)
         opal_shmem_segment_detach (&mca_btl_vader_component.seg_ds);
     }
 
+#if OPAL_BTL_VADER_HAVE_XPMEM
+    if (NULL != mca_btl_vader_component.vma_module) {
+        OBJ_RELEASE(mca_btl_vader_component.vma_module);
+    }
+#endif
+
     return OPAL_SUCCESS;
 }
 
@@ -434,6 +445,7 @@ static struct mca_btl_base_descriptor_t *vader_prepare_src (struct mca_btl_base_
     int rc;
 
     opal_convertor_get_current_pointer (convertor, &data_ptr);
+    assert (NULL != data_ptr);
 
     /* in place send fragment */
     if (OPAL_UNLIKELY(opal_convertor_need_buffers(convertor))) {
@@ -529,14 +541,6 @@ static void mca_btl_vader_endpoint_constructor (mca_btl_vader_endpoint_t *ep)
 }
 
 #if OPAL_BTL_VADER_HAVE_XPMEM
-static int mca_btl_vader_endpoint_rcache_cleanup (mca_mpool_base_registration_t *reg, void *ctx)
-{
-    struct mca_rcache_base_module_t *rcache = (struct mca_rcache_base_module_t *) ctx;
-    /* otherwise dereg will fail on assert */
-    reg->ref_count = 0;
-    (void) rcache->rcache_delete (rcache, reg);
-    return OPAL_SUCCESS;
-}
 #endif
 
 static void mca_btl_vader_endpoint_destructor (mca_btl_vader_endpoint_t *ep)
@@ -546,19 +550,7 @@ static void mca_btl_vader_endpoint_destructor (mca_btl_vader_endpoint_t *ep)
 
 #if OPAL_BTL_VADER_HAVE_XPMEM
     if (MCA_BTL_VADER_XPMEM == mca_btl_vader_component.single_copy_mechanism) {
-        if (ep->segment_data.xpmem.rcache) {
-            /* clean out the registration cache */
-            (void) ep->segment_data.xpmem.rcache->rcache_iterate (ep->segment_data.xpmem.rcache,
-                                                                  NULL, (size_t) -1,
-                                                                  mca_btl_vader_endpoint_rcache_cleanup,
-                                                                  (void *) ep->segment_data.xpmem.rcache);
-            ep->segment_data.xpmem.rcache = NULL;
-        }
-
-        if (ep->segment_base) {
-            xpmem_release (ep->segment_data.xpmem.apid);
-            ep->segment_data.xpmem.apid = 0;
-        }
+        mca_btl_vader_xpmem_cleanup_endpoint (ep);
     } else
 #endif
     if (ep->segment_data.other.seg_ds) {
